\name{getDatasetInfo}
\alias{getDatasetInfo}
\title{Get information about a dataset}
\description{
	Get information about the availability and the default parameters of a dataset.
}
\usage{
getDatasetInfo(dataset, platform, ...);
}
\arguments{
  \item{dataset}{Valid dataset identifier: the GEO or InSilico DB accession code.}
  \item{platform}{The platform of the chip on which a measurement was done.
A platform is identified by its GEO accession code.}
  \item{...}{Optional arguments can be passed to the function for more specific requests:
		\describe{
 			 \item{\code{format = "CURESET"}}{File format in which the dataset should be returned.
			 Possible values:
			\itemize{
				\item{CURESET : Curated R/Bioconductor ExpressionSet.}
				\item{ESET : R/Bioconductor ExpressionSet without curation.}
			}}
  			\item{\code{features = "PROBE"}}{Type of features in which data is represented. Two possible values: "GENE" or "PROBE". By default an expression matrix containing probes is returned. 
  			%Precomputed conversions using genes instead of probes are also available if this argument is set to "GENE". This conversion was made using the \code{nsFilter} function from Bioconductors \pkg{genefilter} package.
  			}
  			\item{\code{norm = "ORIGINAL"}}{Type of normalization to use when pre-processing a dataset. Possible normalizations: "ORIGINAL", "FRMA"[1], "SCAN"[2].}
 			 \item{\code{curation}}{Specific curation used to annotate a dataset.
A curation is identified by a InSilico DB Curation identifier. You can annotate datasets yourself at \url{https://insilicodb.com/app/browse} If not defined, the prefered curation of specified dataset will be used.}
		}  
	}
}
\details{
The function \code{getDatasetInfo} has two functionalities. 

 Its first functionality is to return the default parameters of the given dataset if it is called with dataset and platform as only arguments.  
 
 Its second functionality is to check the availability of a dataset when optional arguments are specified.  Do not forget to surround the function call by a \code{tryCatch} function when checking for availability.
}
\seealso{
Is automatically called before \code{\link{getDataset}}.
}
\examples{
InSilicoLogin("rpackage_tester@insilicodb.com", "5c4d0b231e5cba4a0bc54783b385cc9a");
data.id <- "GSE781";
plt.id <- "GPL96";

## return the default parameters when calling getDataset
eset <- getDatasetInfo( data.id, plt.id);
eset$norm;
## [1] "FRMA"

plt.id <- "GPL97";

## check the availability of the normalizations
norms <- c("FRMA", "ORIGINAL")
output <- sapply(norms, function(n) {
  tryCatch({
    eset <- getDatasetInfo(data.id, plt.id, norm = n);
    eset$norm;
  }, error = function(e) {
    message(e);
  });
});
output$FRMA
## NULL
output$ORIGINAL
## [1] "ORIGINAL"
}
\references{
[1] MN. McCall, BM. Bolstad, and RA. Irizarry, Frozen Robust Multi-Array Analysis (fRMA),
  \emph{Biostatistics}, vol. 11, no. 2, pp. 242-253 , 2008.
  
[2] SR. Piccolo, Y. Sun, JD. Campbell, ME. Lenburg, AH. Bild, and WE. Johnson, A single-sample microarray
  normalization method to facilitate personalized-medicine workflows, \emph{Genomics}, vol. 100, no. 6, pp. 337-344, 2012.
}