\name{getDatasets}
\alias{getDatasets}
\title{Get datasets in ExpressionSet format}
\description{
	Get the datasets for every platform for the given dataset identifier.
}
\usage{
getDatasets(dataset, ...);
}
\arguments{
  \item{dataset}{Valid dataset identifier: the GEO or InSilico DB accession code.}
  \item{...}{Optional arguments can be passed to the function for more specific requests:
  \describe{
 			 \item{\code{format = "CURESET"}}{File format in which the dataset should be returned.
			 Possible values:
			\itemize{
				\item{CURESET : Curated R/Bioconductor ExpressionSet.}
				\item{ESET : R/Bioconductor ExpressionSet without curation.}
			}}
  			\item{\code{features = "PROBE"}}{Type of features in which data is represented. Two possible values: "GENE" or "PROBE". By default an expression matrix containing probes is returned
  			%Precomputed conversions using genes instead of probes are also available if this argument is set to "GENE". This conversion was made using the \code{nsFilter} function from Bioconductors \pkg{genefilter} package.
  			}
  			\item{\code{norm = "ORIGINAL"}}{Type of normalization to use when pre-processing a dataset. Possible normalizations: "ORIGINAL", "FRMA"[1], "SCAN"[2].}
 			 \item{\code{curation}}{Specific curation used to annotate a dataset.
A curation is identified by a InSilico DB Curation identifier. You can annotate datasets yourself at \url{https://insilicodb.com/app/browse} If not defined, the default curation of specified dataset will be used.}   
  			\item{\code{timeout = 120}}{Time (in seconds) to wait for the dataset, if it has to be generated.}
	}
  }
} 
\value{
A list with a Bioconductors ExpressionSet for every platform.
}
\examples{
InSilicoLogin("rpackage_tester@insilicodb.com", "5c4d0b231e5cba4a0bc54783b385cc9a");
dataset.id <- "GSE1456"
getPlatforms(dataset.id);
## [1] "GPL96" "GPL97"
esets <- getDatasets(dataset.id, norm = "ORIGINAL");
sapply(esets, annotation)
##    GPL96    	 GPL97
##    "hgu133a"   "hgu133b"

}
\references{
[1] MN. McCall, BM. Bolstad, and RA. Irizarry, Frozen Robust Multi-Array Analysis (fRMA),
  \emph{Biostatistics}, vol. 11, no. 2, pp. 242-253 , 2008.
  
[2] SR. Piccolo, Y. Sun, JD. Campbell, ME. Lenburg, AH. Bild, and WE. Johnson, A single-sample microarray
  normalization method to facilitate personalized-medicine workflows, \emph{Genomics}, vol. 100, no. 6, pp. 337-344, 2012.
}